import datetime
from bson.objectid import ObjectId
from util import database, util
from api import uploads, objects


def get_users(user):
    db = database.get_db()
    if not util.is_root(user):
        raise util.errors.Forbidden("Not allowed")
    users = list(
        db.users.find(
            {},
            {
                "username": 1,
                "avatar": 1,
                "email": 1,
                "createdAt": 1,
                "lastSeenAt": 1,
                "roles": 1,
                "groups": 1,
            },
        )
        .sort("lastSeenAt", -1)
        .limit(200)
    )
    group_ids = []
    for u in users:
        group_ids += u.get("groups", [])
    groups = list(db.groups.find({"_id": {"$in": group_ids}}, {"name": 1}))
    projects = list(db.projects.find({}, {"name": 1, "path": 1, "user": 1}))
    for u in users:
        if "avatar" in u:
            u["avatarUrl"] = uploads.get_presigned_url(
                "users/{0}/{1}".format(str(u["_id"]), u["avatar"])
            )
        u["projects"] = []
        for p in projects:
            if p["user"] == u["_id"]:
                u["projects"].append(p)
        u["groupMemberships"] = []
        if u.get("groups"):
            for g in groups:
                if g["_id"] in u.get("groups", []):
                    u["groupMemberships"].append(g)
    return {"users": users}


def get_groups(user):
    db = database.get_db()
    if not util.is_root(user):
        raise util.errors.Forbidden("Not allowed")
    groups = list(db.groups.find({}))
    for group in groups:
        group["memberCount"] = db.users.count_documents({"groups": group["_id"]})
    return {"groups": groups}


def get_moderation(user):
    db = database.get_db()
    if not util.is_root(user):
        raise util.errors.Forbidden("Not allowed")
    object_list = list(db.objects.find({"moderationRequired": True}))
    for obj in object_list:
        if obj["type"] == "file" and "storedName" in obj:
            obj["url"] = uploads.get_presigned_url(
                "projects/{0}/{1}".format(obj["project"], obj["storedName"])
            )
    comment_list = list(db.comments.find({"moderationRequired": True}))
    user_list = list(db.users.find({"moderationRequired": True}, {"username": 1}))
    return {"objects": object_list, "comments": comment_list, "users": user_list}


def moderate(user, item_type, item_id, allowed):
    db = database.get_db()
    if not util.is_root(user):
        raise util.errors.Forbidden("Not allowed")
    if item_type not in ["objects", "comments", "users"]:
        raise util.errors.BadRequest("Invalid item type")
    item_id = ObjectId(item_id)
    item = db[item_type].find_one({"_id": item_id})
    # For now, handle only allowed moderations.
    # Disallowed will be manually managed.
    if item and allowed:
        db[item_type].update_one(
            {"_id": item_id},
            {
                "$set": {
                    "moderationRequired": False,
                    "moderated": True,
                    "moderatedAt": datetime.datetime.now(),
                    "moderatedBy": user["_id"],
                }
            },
        )
        if item_type == "comments":
            objects.send_comment_notification(item_id)
    return {"success": True}
