import React, { useState, useEffect } from 'react'
import { Form, Input, TextArea, Modal, Loader, Segment, Card, Button } from 'semantic-ui-react'
import { useSelector } from 'react-redux'
import { Link, useParams } from 'react-router-dom'
import { toast } from 'react-toastify'
import moment from 'moment'
import api from '../../../api'
import utils from '../../../utils/utils.js'
import TrendsImage from '../../../images/trends.png'

export default function Forum () {
  const [topics, setTopics] = useState([])
  const [loadingTopics, setLoadingTopics] = useState(false)
  const [query, setQuery] = useState('')
  const { id } = useParams()

  const { user, group } = useSelector(state => {
    const group = state.groups.groups?.filter(g => g._id === id)[0]
    const user = state.users.users.filter(u => state.auth.currentUserId === u._id)[0]
    return { user, group }
  })

  useEffect(() => {
    setLoadingTopics(true)
    api.groups.getForumTopics(group._id, topics => {
      setLoadingTopics(false)
      setTopics(topics)
    }, err => {
      toast.error(err.message)
      setLoadingTopics(false)
    })
  }, [group._id])

  function deleteTopic (topicId) {
    utils.confirm('Delete topic', 'Really delete this topic? This cannot be undone.').then(() => {
      api.groups.deleteForumTopic(group._id, topicId, () => {
        setTopics(topics.filter(t => t._id !== topicId))
      }, err => {
        toast.error(err.message)
      })
    }).catch(() => {})
  }

  const filteredTopics = query ? topics.filter(t => t.title.toLowerCase().includes(query.toLowerCase())) : topics

  return (
    <div>
      <h1>Forum</h1>
      <div style={{ display: 'flex', justifyContent: 'space-between', marginBottom: 20 }}>
        <Input placeholder='Search topics...' value={query} onChange={e => setQuery(e.target.value)} />
        {utils.hasGroupPermission(user, group, 'postForumTopics') &&
          <TopicCreator
            trigger={<Button color='teal'>Create a topic</Button>}
            onCreateTopic={newTopic => setTopics([newTopic, ...topics])}
          />}
      </div>
      {loadingTopics
        ? <Loader active />
        : (
            topics.length === 0
              ? (
                <Segment placeholder textAlign='center'>
                  <img src={TrendsImage} alt='No topics' style={{ maxWidth: 300, display: 'block', margin: '10px auto' }} />
                  <h3>Forum topics</h3>
                  <p>This group doesn't yet have any forum topics.</p>
                </Segment>)
              : (
                <Card.Group>
                  {filteredTopics?.map(topic => (
                    <Card key={topic._id}>
                      <Card.Content as={Link} to={`/groups/${group._id}/forum/topics/${topic._id}`}>
                        <Card.Header>{topic.title}</Card.Header>
                        <Card.Description>{topic.description}</Card.Description>
                      </Card.Content>
                      {topic.totalReplies > 0 &&
                        <Card.Content extra>
                          {topic.totalReplies} {topic.totalReplies === 1 ? 'reply' : 'replies'} (last {moment(topic.lastReplyAt).fromNow()})
                        </Card.Content>}
                      {(utils.isGroupAdmin(user, group) || topic.user === user._id) &&
                        <Card.Content extra>
                          <TopicCreator
                            topic={topic}
                            trigger={<Button size='tiny' basic>Edit</Button>}
                            onCreateTopic={newTopic => setTopics(topics.map(t => t._id === newTopic._id ? newTopic : t))}
                          />
                          <Button size='tiny' basic onClick={() => deleteTopic(topic._id)} color='orange'>Delete</Button>
                        </Card.Content>}
                    </Card>
                  ))}
                </Card.Group>))}
    </div>
  )
}

function TopicCreator ({ topic, trigger, onCreateTopic }) {
  const [title, setTitle] = useState(topic?.title || '')
  const [description, setDescription] = useState(topic?.description || '')
  const [open, setOpen] = useState(false)
  const { id } = useParams()

  function createTopic () {
    const successCb = newTopic => {
      onCreateTopic && onCreateTopic(newTopic)
      setOpen(false)
    }
    const errorCb = err => {
      toast.error(err.message)
    }
    if (topic) {
      api.groups.updateForumTopic(id, topic._id, { title, description }, successCb, errorCb)
    } else {
      api.groups.createForumTopic(id, { title, description }, successCb, errorCb)
    }
  }

  return (
    <>
      {trigger && React.cloneElement(trigger, { onClick: () => setOpen(true) })}
      <Modal open={open} onClose={() => setOpen(false)}>
        <Modal.Header>{topic ? 'Edit' : 'New'} Topic</Modal.Header>
        <Modal.Content>
          <Form>
            <Input placeholder='Title' value={title} onChange={e => setTitle(e.target.value)} />
            <div style={{ marginTop: 10 }}>
              <TextArea placeholder='Write a brief description...' value={description} onChange={e => setDescription(e.target.value)} />
            </div>
          </Form>
        </Modal.Content>
        <Modal.Actions>
          <Button onClick={() => setOpen(false)}>Cancel</Button>
          <Button primary onClick={createTopic}>{topic ? 'Save' : 'Create'}</Button>
        </Modal.Actions>
      </Modal>
    </>
  )
}
