import React, { useState, useEffect } from 'react'
import { Loader, Segment, Button } from 'semantic-ui-react'
import { useSelector, useDispatch } from 'react-redux'
import { Link, useParams, useLocation } from 'react-router-dom'
import { toast } from 'react-toastify'
import actions from '../../../actions'
import api from '../../../api'
import utils from '../../../utils/utils'
import UserChip from '../../includes/UserChip'
import NewFeedMessage from '../../includes/NewFeedMessage'
import FormattedMessage from '../../includes/FormattedMessage'
import ReportLink from '../../includes/ReportLink'
import StartChatImage from '../../../images/startchat.png'

export default function ForumTopic () {
  const [topic, setTopic] = useState(null)
  const [replies, setReplies] = useState([])
  const [totalReplies, setTotalReplies] = useState(0)
  const [totalPages, setTotalPages] = useState(1)
  const [loadingReplies, setLoadingReplies] = useState(false)
  const { hash } = useLocation()
  const { id, topicId, page: pageParam } = useParams()
  const dispatch = useDispatch()
  const page = parseInt(pageParam) || 1

  const { user, group } = useSelector(state => {
    const group = state.groups.groups.filter(g => g._id === id)[0]
    const user = state.users.users.filter(u => state.auth.currentUserId === u._id)[0]
    return { user, group }
  })

  useEffect(() => {
    setLoadingReplies(true)
    api.groups.getForumTopicReplies(group._id, topicId, page, ({ topic, replies, totalReplies, totalPages }) => {
      setLoadingReplies(false)
      setTopic(topic)
      setReplies(replies)
      setTotalReplies(totalReplies)
      setTotalPages(totalPages)
    }, err => {
      toast.error(err.message)
      setLoadingReplies(false)
    })
  }, [group._id, topicId, page])

  useEffect(() => {
    if (hash === '#forum-reply') {
      setTimeout(() => {
        const element = document.getElementById('forum-reply')
        element.scrollIntoView({ behavior: 'smooth' })
      }, 400)
    }
  }, [hash])

  const toggleEmailSub = (key, enable) => {
    if (enable) { api.users.createEmailSubscription(user.username, key, ({ subscriptions }) => dispatch(actions.users.updateSubscriptions(user._id, subscriptions)), err => toast.error(err.message)) } else { api.users.deleteEmailSubscription(user.username, key, ({ subscriptions }) => dispatch(actions.users.updateSubscriptions(user._id, subscriptions)), err => toast.error(err.message)) }
  }

  function handleDeleteReply (replyId) {
    utils.confirm('Delete reply', 'Really delete this reply? This cannot be undone.').then(() => {
      api.groups.deleteForumTopicReply(group._id, topicId, replyId, () => {
        setReplies(replies.filter(r => r._id !== replyId))
        setTotalReplies(totalReplies - 1)
      }, err => {
        toast.error(err.message)
      })
    }).catch(() => {})
  }

  const BASE_ROUTE = `/groups/${id}/forum/topics/${topicId}`

  return (
    <div>
      <Button basic as={Link} to={`/groups/${id}/forum`}>View all topics</Button>
      <div style={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', marginTop: 20 }}>
        <h2>{topic?.title || 'Topic'}</h2>
        <div>
          {utils.hasEmailSubscription(user, `groupForumTopic-${topic?._id}`)
            ? <Button color='blue' basic size='tiny' icon='check' content='Subscribed to email updates' onClick={e => toggleEmailSub(`groupForumTopic-${topic?._id}`, false)} />
            : <Button color='blue' size='tiny' icon='rss' content='Subscribe to updates' onClick={e => toggleEmailSub(`groupForumTopic-${topic?._id}`, true)} />}
          {utils.hasGroupPermission(user, group, 'postForumTopicReplies') && totalPages > 0 &&
            <Button
              color='teal' size='small' as={Link} to={`${BASE_ROUTE}/page/${totalPages}#forum-reply`} onClick={e => {
                if (page === totalPages) {
                  e.preventDefault()
                  document.querySelector('#forum-reply textarea').focus()
                }
              }}
            >
              Write a reply to this topic
            </Button>}
        </div>
      </div>
      {loadingReplies && <Loader active />}
      {!loadingReplies && replies.length === 0 &&
        <Segment placeholder textAlign='center'>
          <img src={StartChatImage} alt='Start chatting' style={{ width: 200, margin: '20px auto' }} />
          <p>This topic doesn't have any replies yet.</p>
        </Segment>}

      {replies.map(reply => (
        <Segment key={reply._id}>
          <div style={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', marginBottom: 20 }}>
            <UserChip user={reply.author} compact />
            <div style={{ fontSize: 12, color: 'rgb(150,150,150)', display: 'flex', alignItems: 'center' }}>
              <div>{new Date(reply.createdAt).toLocaleString()}</div>
              {(utils.isGroupAdmin(user, group) || user._id === reply.author._id) &&
                <div role='button' style={{ textDecoration: 'underline', cursor: 'pointer', marginLeft: 10 }} onClick={() => handleDeleteReply(reply._id)}>Delete</div>}
              <ReportLink referrer={`/topicReplies/${reply._id}`} style={{ fontSize: 11 }} marginLeft='10px' />
            </div>
          </div>
          <FormattedMessage content={reply.content} />
          {reply.attachments && reply.attachments.length > 0 &&
            <div>
              <div style={{ display: 'flex', flexWrap: 'wrap', marginTop: 10 }}>
                {reply.attachments.filter(a => a.isImage).map(a =>
                  <a key={a.storedName} href={a.url} target='blank' rel='noopener noreferrer'><div style={{ width: 100, height: 100, background: `url(${a.url}) center center / cover`, marginRight: 10, borderRadius: 10 }} /></a>)}
              </div>
              <div style={{ marginTop: 10, display: 'flex', flexWrap: 'wrap' }}>
                {reply.attachments.map(a => {
                  if (a.type === 'project') {
                    return <Button key={a.fullName} as={Link} to={`/${a.fullName}`} size='tiny' icon='book' content={a.name} style={{ margin: 4 }} />
                  } else if (a.type === 'file' && !a.isImage) {
                    return <Button key={a.url} as='a' href={a.url} target='blank' rel='noopener noreferrer' basic compact size='tiny' style={{ margin: 5 }} icon='download' content={a.name} />
                  }
                  return null
                })}
              </div>
            </div>}
        </Segment>
      ))}

      {totalPages > 1 &&
        <div style={{ display: 'flex', justifyContent: 'center', alignItems: 'center', marginTop: 20, marginBottom: 20 }}>
          <Button size='tiny' disabled={page === 1} as={Link} to={`${BASE_ROUTE}/page/${page - 1}`}>Previous page</Button>
          <span style={{ margin: '0 10px' }}>Page {page} of {totalPages}</span>
          <Button size='tiny' disabled={page === totalPages} as={Link} to={`${BASE_ROUTE}/page/${page + 1}`}>Next page</Button>
        </div>}

      {utils.hasGroupPermission(user, group, 'postForumTopicReplies') &&
        (page >= totalPages
          ? (
            <div id='forum-reply'>
              <NewFeedMessage
                user={user} group={group} topic={topic} forType='groupForum' onPosted={newReply => {
                  setTotalReplies(totalReplies + 1)
                  setReplies([...replies, newReply])
                  toggleEmailSub(`groupForumTopic-${topic?._id}`, true)
                }}
              />
            </div>)
          : (
            <div style={{ display: 'flex', justifyContent: 'center', alignItems: 'center', marginTop: 20 }}>
              <Button color='teal' as={Link} to={`${BASE_ROUTE}/page/${totalPages}#forum-reply`}>Write a reply to this topic</Button>
            </div>))}

    </div>
  )
}
