import React from 'react'
import { Header, Button, Divider, Segment, Form, Checkbox } from 'semantic-ui-react'
import { useNavigate, useParams } from 'react-router-dom'
import { useSelector, useDispatch } from 'react-redux'
import { toast } from 'react-toastify'
import utils from '../../../utils/utils.js'
import actions from '../../../actions'
import api from '../../../api'
import FileChooser from '../../includes/FileChooser'

const PERMISSIONS = [
  { name: 'viewMembers', label: 'Allow members to view other members' },
  { name: 'viewNoticeboard', label: 'Allow members to view the noticeboard' },
  { name: 'postNoticeboard', label: 'Allow members to post to the noticeboard' },
  { name: 'viewProjects', label: 'Allow members to view projects linked to the group' },
  { name: 'viewForumTopics', label: 'Allow members to view forum topics' },
  { name: 'postForumTopics', label: 'Allow members to post new topics to the forum' },
  { name: 'postForumTopicReplies', label: 'Allow members to reply to forum topics' }
]

function Settings () {
  const navigate = useNavigate()
  const dispatch = useDispatch()
  const { id } = useParams()

  const { loading, group } = useSelector(state => {
    const { loading } = state.groups
    const group = state.groups.groups.filter(g => g._id === id)[0]
    return { loading, group }
  })

  const saveGroup = () => {
    dispatch(actions.groups.request(true))
    const { _id, name, description, closed, advertised } = group
    api.groups.update(_id, { name, description, closed, advertised }, () => {
      dispatch(actions.groups.request(false))
      toast.info('Group updated')
    }, err => {
      toast.error(err.message)
      dispatch(actions.groups.request(false))
    })
  }

  const savePermission = (permissionName, enabled) => {
    const permissions = group.memberPermissions || []
    const index = permissions.indexOf(permissionName)
    if (enabled && index === -1) {
      permissions.push(permissionName)
    } else if (!enabled && index !== -1) {
      permissions.splice(index, 1)
    }
    api.groups.update(group._id, { memberPermissions: permissions }, g => {
      dispatch(actions.groups.updateGroup(group._id, { memberPermissions: g.memberPermissions }))
    }, err => {
      toast.error(err.message)
    })
  }

  const deleteGroup = () => {
    utils.confirm('Really delete this group?', 'You\'ll lose all entries in the group feed and anything else you\'ve added to it.').then(() => {
      api.groups.delete(group._id, () => {
        toast.info('Group deleted')
        dispatch(actions.groups.deleteGroup(group._id))
        navigate('/')
      }, err => toast.error(err.message))
    }, () => {})
  }

  const updatePicture = (image) => {
    api.groups.update(group._id, { image }, g => {
      if (!image) { // Needed to ensure the avatar is immediately unset
        g.image = null
        g.imageUrl = null
      }
      dispatch(actions.groups.updateGroup(group._id, { image: g.image, imageUrl: g.imageUrl }))
    })
  }

  return (
    <div>
      <Segment color='blue'>
        <Header>About this group</Header>
        <Form>
          <Form.Input label='Group name' value={group.name} onChange={e => dispatch(actions.groups.updateGroup(group._id, { name: e.target.value }))} />
          <Form.TextArea label='Group description' value={group.description} onChange={e => dispatch(actions.groups.updateGroup(group._id, { description: e.target.value }))} />
          <Form.Checkbox toggle checked={group.closed} label='Closed group' onChange={(e, c) => dispatch(actions.groups.updateGroup(group._id, { closed: c.checked }))} />
          <div style={{ marginLeft: 63, color: 'rgb(150,150,150)', marginBottom: 20 }}>
            <p>In a closed group, new members must be invited or approved to join.</p>
          </div>
          <Form.Checkbox toggle checked={group.advertised} label='Publicise this group' onChange={(e, c) => dispatch(actions.groups.updateGroup(group._id, { advertised: c.checked }))} />
          <div style={{ marginLeft: 63, color: 'rgb(150,150,150)' }}>
            <p>If a group is publicised, it may allow other people to discover the group.</p>
          </div>

          <Divider hidden />
          <Form.Button loading={loading} color='teal' icon='check' content='Save changes' onClick={saveGroup} />
        </Form>

        <Header>Group picture</Header>
        <p>Upload a picture to represent your group. This will be shown on the group page and may be viewable to both members and non-members.</p>
        {group.imageUrl &&
          <div style={{ marginBottom: 10 }}>
            <img src={group.imageUrl} alt='Group' style={{ maxWidth: 200, borderRadius: 5 }} />
          </div>}

        <div style={{ display: 'flex', alignItems: 'center' }}>
          {group.imageUrl &&
            <Button basic color='gray' icon='times' content='Remove image' onClick={() => updatePicture(null)} />}
          <FileChooser
            forType='group' forObject={group}
            trigger={<Button basic color='yellow' icon='image' content='Choose an image' />}
            accept='image/*' onComplete={f => updatePicture(f.storedName)}
          />
        </div>
      </Segment>

      <Segment color='blue'>
        <Header>Permissions</Header>
        <p>Group admins can always perform all group actions.</p>

        {PERMISSIONS.map(p => (
          <div key={p.name} style={{ marginBottom: 10 }}>
            <Checkbox toggle label={p.label} onChange={(e, c) => savePermission(p.name, c.checked)} checked={group.memberPermissions?.includes(p.name)} />
          </div>
        ))}
      </Segment>

      <Segment color='red'>
        <Header>Delete this group</Header>
        <p>This action will also delete all feed entries and anything else you've added. We'll remove all members and administrators from the group and then safely delete it.</p>
        <Button color='red' icon='trash' content='Delete group' onClick={deleteGroup} />
      </Segment>
    </div>
  )
}

export default Settings
